@@ -31,31 +31,6 @@ void fitch_init(int *data, int *m, int *n, double *weights, int *nr)
 }
 
 
-SEXP getData(SEXP n, SEXP k){
-    int i, m=INTEGER(n)[0], l=INTEGER(k)[0];  
-    SEXP DAT, DAT2, RESULT;
-    PROTECT(RESULT = allocVector(VECSXP, 2L));
-    PROTECT(DAT = allocMatrix(INTSXP, m, l));
-    PROTECT(DAT2 = allocMatrix(INTSXP, m, l)); 
-    for(i=0; i< m*l; i++) INTEGER(DAT)[i] = data1[i];
-    for(i=0; i< m*l; i++) INTEGER(DAT2)[i] = data2[i];
-    SET_VECTOR_ELT(RESULT, 0, DAT);
-    SET_VECTOR_ELT(RESULT, 1, DAT2);
-    UNPROTECT(3);
-    return(RESULT); 
-}
-
-
-SEXP getWeight(SEXP n){
-    int i, m=INTEGER(n)[0];  
-    SEXP RESULT;
-    PROTECT(RESULT = allocVector(REALSXP, m));
-    for(i=0; i<m; i++) REAL(RESULT)[i] = weight[i];
-    UNPROTECT(1);
-    return(RESULT); 
-}
-
-
 int bitcount(int x){ 
     int count;
     for (count=0; x != 0; x>>=1)
@@ -183,7 +158,6 @@ if(*nthreads <= 1){ *nthreads=1; }else{ *nthreads=(*nthreads < omp_get_max_threa
             if(!tmp){
                 tmp = data1[k + ei*nr] | data2[k + ei*nr];
                 pvtmp[i]+=weight[k];
-                
             }
             tmp = tmp & data1[k + d3*nr];
             if(!tmp){
@@ -356,6 +330,7 @@ void fitchNNN(int d1, int d2){
     if(tmp) d1 = tmp;
     else d1 = d1 | d2;
 }
+
 // haeufig 0
 void fitchTripletNew(int *res, int *dat1, int *dat2, int *dat3, int *nr) 
 {   
@@ -389,13 +364,6 @@ void fitchN(int *dat1, int *dat2, int *nr){
     } 
 }
 
-// raus
-void fitchN2(int *res, int *dat, int *node, int *edge, int *nr, int *nl) { 
-    int i;
-    for (i=0; i< *nl; i++) {
-        fitchN(&res[(node[i]-1L) * (*nr)], &dat[(edge[i]-1L) * (*nr)], nr);              
-    }
-}
 
 // MPR reconstruction nicht immer gleiches ergebnis
 void fitchTriplet(int *res, int *dat1, int *dat2, int *dat3, int *nr) 
@@ -444,6 +412,28 @@ void C_MPR(int *res, int *nr, int *parent, int *kids, int *nl) {
 }
 
 
+SEXP C_MPR2(SEXP nrx, SEXP PARENT, SEXP KIDS, SEXP nlx, SEXP M) { 
+    int nr=INTEGER(nrx)[0], nl=INTEGER(nlx)[0], m=INTEGER(M)[0], *res;
+    int *parent = INTEGER(PARENT), *kids=INTEGER(KIDS);
+    int j, p, k1, k2;
+    int i = nl -1;    
+    SEXP RES;
+    PROTECT(RES = allocVector(INTSXP, nr * m)); 
+    res = INTEGER(RES);
+    for(j = 0; j < (nr * m); j++) res[j]=0;
+    while (i > 0L) {
+        p = parent[i] - 1L;
+        k1 = kids[i] - 1L;
+        k2 = kids[i-1L] - 1L;
+        fitchTripletNew(&res[nr * p], &data1[nr * k1], &data1[nr * k2], &data2[nr * p], &nr);
+        i -= 2L;
+    }     
+    UNPROTECT(1);
+    return(RES);
+}
+
+
+
 void fitchNACC2(int *root, int *dat, int *nr, double *pars, int *result, double *weight, double *pars1){
     int k;
     int tmp;
@@ -532,10 +522,6 @@ SEXP FITCH345(SEXP nrx, SEXP node, SEXP edge, SEXP l, SEXP mx, SEXP ps){
     else return(pars); 
 }
 
-//, double *pvec
-
-
-
 
 void FN4(int *dat, int *res, int *nr, int *node, int *edge, int *nl, int *pc, double *weight, double *tmpvec, double *pvec) { 
     int i=0L, ni, le, ri;
@@ -549,7 +535,7 @@ void FN4(int *dat, int *res, int *nr, int *node, int *edge, int *nl, int *pc, do
         }    
         else{ 
             pvec[ni] = tmpvec[le] + pvec[ri];
-	        fitch54(&res[ni * (*nr)], &dat[le * (*nr)], &res[ri * (*nr)], nr, weight, &pvec[ni]);   
+	          fitch54(&res[ni * (*nr)], &dat[le * (*nr)], &res[ri * (*nr)], nr, weight, &pvec[ni]);   
         }
         i++;
         i++;
@@ -637,14 +623,14 @@ SEXP FNALL_NNI(SEXP nrx, SEXP node, SEXP edge, SEXP l, SEXP mx, SEXP my, SEXP ro
     fnhelp(INTEGER(node), INTEGER(edge),  n, &m, INTEGER(root), edge2, node2, pc); 
     fitch9(data1, nr, INTEGER(node), INTEGER(edge), INTEGER(l), weight, pvtmp, &pscore); 
     FN4(data1, data2, nr, node2, edge2, INTEGER(my), pc, weight, pvtmp, pvtmp2); // pars,
-    
+//  fitchQuartet(int *index, int *n, nr, double *psc1, double *psc2, weight, double *res);  
     SET_VECTOR_ELT(res, 0, pvec1);
     SET_VECTOR_ELT(res, 1, pvec2);    
     UNPROTECT(3);
     return(res); 
 }
 
-
+// mpr2 fnodesNew5
 SEXP FNALL5(SEXP nrx, SEXP node, SEXP edge, SEXP l, SEXP mx, SEXP my, SEXP root){   
     int *nr=INTEGER(nrx), m=INTEGER(mx)[0], i,  *n=INTEGER(l);  //*pars,
     double *pvtmp, *pvtmp2, pscore=0.0;  
@@ -655,13 +641,11 @@ SEXP FNALL5(SEXP nrx, SEXP node, SEXP edge, SEXP l, SEXP mx, SEXP my, SEXP root)
     edge2 = (int *) R_alloc(2L * *n, sizeof(int));
     node2 = (int *) R_alloc(2L * *n, sizeof(int));
     pc = (int *) R_alloc(2L * *n, sizeof(int));
-    
-//    pars = (int *) R_alloc(*nr, sizeof(int)); // raus     
+      
     pvtmp2 = (double *) R_alloc(m, sizeof(double));
-    
-    PROTECT(pvec = allocVector(REALSXP, m));
-    
+    PROTECT(pvec = allocVector(REALSXP, m));    
     pvtmp = REAL(pvec);
+    
     for(i=0; i<m; i++){
         pvtmp[i] = 0.0;
         pvtmp2[i] = 0.0;
@@ -699,7 +683,7 @@ void fitchquartet(int *dat1, int *dat2, int *dat3, int *dat4, int *nr, double *w
     }
 }
 
-// weight raus  double *weight,
+// weight raus 
 void fitchQuartet(int *index, int *n, int *nr, double *psc1, double *psc2, double *weight, double *res){
     int i, e1, e2, e3, e4;
     for(i=0; i<*n; i++){ 
